# preparing sample data for limb correction coefficient regression training
# usage: python prep-sample ${Channel No.}
#        where ${Channel No.}=[1, 2, ..., 13]

import numpy as np
import h5py as h5
from glob import glob
from sys import argv

nfov=90
ichan=int(argv[1])

# gathering obs throughout a month into single arrays
outname='AVE-CH{:02d}-201408.h5'.format(ichan)
fnames=sorted(glob('../../stitch/Output/MWTS-201408*'))
lat=np.empty([0,nfov], np.float32)
lon=np.empty([0,nfov], np.float32)
land=np.empty([0,nfov], bool)
obs=np.empty([0,nfov], np.float32)
for fname in fnames:
    print fname
    with h5.File(fname, 'r') as infile:
        lat=np.concatenate((lat, infile['lat']))
        lon=np.concatenate((lon, infile['lon']))
        land=np.concatenate((land, infile['land'][()]))
        obs=np.concatenate((obs, infile['obs'][:,:,ichan-1]))

flagout=obs>320
flagout=np.logical_or(flagout, np.absolute(lat)>90)
flagout=np.logical_or(flagout, np.absolute(lon)>180)
if ichan<=6: flagout=np.logical_or(flagout, land)

obs=np.ma.array(obs, mask=flagout)

# FOV mean TB
TBj=obs.mean(axis=0)

# mean TB within a 2 deg latitude band
latgrid=2.
latbins=np.arange(-89, 90, latgrid)
TBlat=np.ones([latbins.shape[0], nfov], np.float32)*-9999
TBn  =np.ones([latbins.shape[0]], np.float32)*-9999
for ifov in range(nfov):
    for ibin, ilat in enumerate(latbins):
        latsample=lat[:,ifov]
        latflag=np.logical_and(ilat-latgrid/2.<latsample,
                               latsample<ilat+latgrid/2.)
        
        TBlat[ibin,ifov]=obs[:,ifov][latflag].mean()

# nadir temperature
for ibin, ilat in enumerate(latbins):
    latsample=lat[:,[47,48]]
    latflag=np.logical_and(ilat-latgrid/2.<latsample,
                           latsample<ilat+latgrid/2.)
    TBn[ibin]=obs[:,[47,48]][latflag].mean()

TBj[np.isnan(TBj)]=-9999
TBn[np.isnan(TBn)]=-9999
TBlat[np.isnan(TBlat)]=-9999

with h5.File(outname, 'w') as outfile:
    dset=outfile.create_dataset('TBj', data=TBj)
    dset=outfile.create_dataset('TBn', data=TBn)
    dset=outfile.create_dataset('TBlat', data=TBlat)
