"""
Tian, X., X. Zou, and S. Yang, 2018: A limb correction method for the 
Microwave Temperature Sounder 2 and its applications. Adv. Atmos. Sci., 
35(12), 1547–1552, doi: 10.1007/s00376-018-8092-8.
"""

import numpy as np
import h5py as h5
from datetime import datetime, timedelta

nchan=13
nfov=90
def predictor_gen():
    res=[]
    with open('pred-mwts.txt', 'r') as infile:
        for line in infile:
            res.append(np.array(line.strip().split()).astype(np.int8))
    for ichan in range(nchan): res[ichan]-=1
    return res

predictors=predictor_gen()
pred=np.ones([3,nchan], np.int8)*-99
for ichan in range(nchan): 
    npred=len(predictors[ichan])
    pred[:npred,ichan]=predictors[ichan]
    
TBj=[]
TBn=[]
TBlat=[]
for ichan in range(nchan):
    fname='./sample/AVE-Ch{:02d}-201408.h5'.format(ichan+1)
    with h5.File(fname, 'r') as infile:
        TBj.append(infile['TBj'][()])
        TBn.append(infile['TBn'][()])
        TBlat.append(infile['TBlat'][()])
TBj=np.array(TBj).T
TBn=np.array(TBn).T
TBlat=np.swapaxes(np.array(TBlat).T, 0, 1)

latgrid=2.
latbins=np.arange(-89, 90, latgrid)

# linear regression
from sklearn.linear_model import LinearRegression
lr=LinearRegression(fit_intercept=True)

coef=np.ones([3,nfov,nchan], np.float32)*-9999
cept=np.ones([  nfov,nchan], np.float32)*-9999
for ichan in range(nchan): 
    npred=(pred[:,ichan]>=0).sum()
    x=np.zeros([latbins.shape[0], npred], np.float32)
    y=TBn[:,ichan]

    flagin=y>-9000
    for ifov in range(nfov):
        for i in range(npred):
            x[:,i]=TBlat[:,ifov,pred[i,ichan]]-TBj[ifov,pred[i,ichan]]
            flagin=np.logical_and(flagin, x[:,i]>-9000)

        res=lr.fit(x[flagin,:], y[flagin])

        coef[:npred,ifov,ichan]=res.coef_
        cept[ifov,ichan]  =res.intercept_

with h5.File('Limb-Coef-201408.h5', 'w') as outfile:
    dset=outfile.create_dataset('predictors', data=pred)
    dset=outfile.create_dataset('coef', data=coef)
    dset=outfile.create_dataset('intercept', data=cept)
    dset=outfile.create_dataset('TB_FOV_mean', data=TBj)
